import { IncomingMessage, ServerResponse } from 'http';
import * as nookies from 'nookies';
import * as crypto from 'crypto';

const COMMON_COOKIE_CONFIG = {
	path: '/',
	httpOnly: true,
};

const COMMON_SET_COOKIE_CONFIG = {
	...COMMON_COOKIE_CONFIG,
	maxAge: 30 * 24 * 60 * 60,
};

const cookieKeys: Record<string, string> = {};

export const REQUEST_ID_COOKIE_KEY = 'b' as const;
export class CookieManager {
	private readonly ctx: { req: IncomingMessage, res: ServerResponse<IncomingMessage> };

	constructor(ctx: { req: IncomingMessage, res: ServerResponse<IncomingMessage> }) {
		// noop
		this.ctx = ctx;
	}

	private static generateCookieKey(key: string) {
		const random = crypto.randomBytes(16).toString('hex');
		return `if${key}${random}`;
	}

	setCookie(key: string, value: string) {
		// cleanup previous cookie
		this.unsetCookie(key);
		cookieKeys[key] = CookieManager.generateCookieKey(key);
		nookies.setCookie(
			this.ctx,
			cookieKeys[key],
			value,
			COMMON_SET_COOKIE_CONFIG,
		);
	}

	unsetCookie(key: string) {
		nookies.destroyCookie(this.ctx, cookieKeys[key], COMMON_COOKIE_CONFIG);
	}

	hasCookie(key: string) {
		const cookies = nookies.parseCookies(this.ctx);
		return cookieKeys[key] in cookies;
	}

	getCookie(key: string) {
		const cookies = nookies.parseCookies(this.ctx);
		return cookies[cookieKeys[key]];
	}
}
